@@ -1,7 +1,10 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  *  Copyright © 2002 Jorn Baayen
  *  Copyright © 2003 Marco Pesenti Gritti
  *  Copyright © 2003, 2004 Christian Persch
+ *  Copyright © 2009 Igalia S.L.
+ *  Copyright © 2009 Holger Hans Peter Freyther
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,41 +18,38 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: pdm-dialog.c,v 1.63.2.2 2006/09/13 11:48:54 chpe Exp $
  */
 
 #include "config.h"
 
 #include "pdm-dialog.h"
 #include "ephy-shell.h"
-#include "ephy-cookie-manager.h"
 #include "ephy-file-helpers.h"
-#include "ephy-password-manager.h"
 #include "ephy-gui.h"
 #include "ephy-state.h"
 #include "ephy-string.h"
+#include "ephy-stock-icons.h"
 #include "ephy-debug.h"
+#include "ephy-time-helpers.h"
+#include "ephy-embed-single.h"
+#include "ephy-favicon-cache.h"
+#include "ephy-history.h"
+#include "ephy-password-info.h"
 
-#include <gtk/gtklabel.h>
-#include <gtk/gtkbox.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtktreeselection.h>
-#include <gtk/gtkdialog.h>
-#include <gtk/gtktreeview.h>
-#include <gtk/gtkliststore.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtknotebook.h>
-#include <gtk/gtktogglebutton.h>
+#include <gtk/gtk.h>
 #include <glib/gi18n.h>
+#include <libsoup/soup.h>
+#include <webkit/webkit.h>
+#include <gnome-keyring.h>
+#include <gnome-keyring-memory.h>
 
-#include <time.h>
 #include <string.h>
+#include <time.h>
 
 typedef struct PdmActionInfo PdmActionInfo;
-	
+
 struct PdmActionInfo
 {
 	/* Methods */
@@ -74,6 +74,13 @@ struct PdmActionInfo
 	gboolean delete_row_on_remove;
 };
 
+typedef struct PdmCallBackData PdmCallBackData;
+struct PdmCallBackData
+{
+    guint key;
+    GtkListStore *store;
+};
+
 #define EPHY_PDM_DIALOG_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_PDM_DIALOG, PdmDialogPrivate))
 
 struct PdmDialogPrivate
@@ -103,7 +110,7 @@ enum
 	COL_PASSWORDS_HOST,
 	COL_PASSWORDS_USER,
 	COL_PASSWORDS_PASSWORD,
-	COL_PASSWORDS_DATA
+	COL_PASSWORDS_DATA,
 };
 
 enum
@@ -118,18 +125,23 @@ enum
 	PROP_PASSWORDS_SHOW
 };
 
+enum
+{
+	PDM_DIALOG_RESPONSE_CLEAR = 1
+};
+
 static const
 EphyDialogProperty properties [] =
 {
 	{ "pdm_dialog",			NULL, PT_NORMAL, 0 },
 	{ "pdm_notebook",		NULL, PT_NORMAL, 0 },
 
-	{ "cookies_treeview",	   	NULL, PT_NORMAL, 0 },
-	{ "cookies_remove_button",     	NULL, PT_NORMAL, 0 },
-	{ "cookies_properties_button", 	NULL, PT_NORMAL, 0 },
-	{ "passwords_treeview",	       	NULL, PT_NORMAL, 0 },
-	{ "passwords_remove_button",   	NULL, PT_NORMAL, 0 },
-	{ "passwords_show_button",     	NULL, PT_NORMAL, 0 },
+	{ "cookies_treeview",		NULL, PT_NORMAL, 0 },
+	{ "cookies_remove_button",	NULL, PT_NORMAL, 0 },
+	{ "cookies_properties_button",	NULL, PT_NORMAL, 0 },
+	{ "passwords_treeview",		NULL, PT_NORMAL, 0 },
+	{ "passwords_remove_button",	NULL, PT_NORMAL, 0 },
+	{ "passwords_show_button",	NULL, PT_NORMAL, 0 },
 
 	{ NULL }
 };
@@ -137,44 +149,15 @@ EphyDialogProperty properties [] =
 static void pdm_dialog_class_init	(PdmDialogClass *klass);
 static void pdm_dialog_init		(PdmDialog *dialog);
 static void pdm_dialog_finalize		(GObject *object);
+static void pdm_dialog_password_remove (PdmActionInfo *info, gpointer data);
 
-static GObjectClass *parent_class = NULL;
-
-GType 
-pdm_dialog_get_type (void)
-{
-	static GType type = 0;
-
-	if (G_UNLIKELY (type == 0))
-	{
-		const GTypeInfo our_info =
-		{
-			sizeof (PdmDialogClass),
-			NULL, /* base_init */
-			NULL, /* base_finalize */
-			(GClassInitFunc) pdm_dialog_class_init,
-			NULL,
-			NULL, /* class_data */
-			sizeof (PdmDialog),
-			0, /* n_preallocs */
-			(GInstanceInitFunc) pdm_dialog_init
-		};
-
-		type = g_type_register_static (EPHY_TYPE_DIALOG,
-					       "PdmDialog",
-					       &our_info, 0);
-	}
-
-	return type;
-}
+G_DEFINE_TYPE (PdmDialog, pdm_dialog, EPHY_TYPE_DIALOG)
 
 static void
 pdm_dialog_class_init (PdmDialogClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->finalize = pdm_dialog_finalize;
 
 	g_type_class_add_private (object_class, sizeof(PdmDialogPrivate));
@@ -200,7 +183,299 @@ pdm_dialog_show_help (PdmDialog *pd)
 	id = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
 	g_return_if_fail (id == 0 || id == 1);
 
-	ephy_gui_help (GTK_WINDOW (window), "epiphany", help_preferences[id]);
+	ephy_gui_help (window, help_preferences[id]);
+}
+
+typedef struct
+{
+	EphyDialog *dialog;
+	GtkWidget *checkbutton_history;
+	GtkWidget *checkbutton_cookies;
+	GtkWidget *checkbutton_passwords;
+	GtkWidget *checkbutton_cache;
+	guint num_checked;
+} PdmClearAllDialogButtons;
+
+static SoupCookieJar*
+get_cookie_jar ()
+{
+	SoupSession* session;
+
+	session = webkit_get_default_session ();
+	return (SoupCookieJar*)soup_session_get_feature (session, SOUP_TYPE_COOKIE_JAR);
+}
+
+static void
+clear_all_dialog_release_cb (PdmClearAllDialogButtons *data)
+{
+	g_slice_free (PdmClearAllDialogButtons, data);
+}
+
+static void
+clear_all_cookies (SoupCookieJar *jar)
+{
+	GSList *l, *p;
+
+	l = soup_cookie_jar_all_cookies (jar);
+	for (p = l; p; p = p->next)
+		soup_cookie_jar_delete_cookie (jar, (SoupCookie*)p->data);
+
+	soup_cookies_free (l);
+}
+
+static void
+pdm_dialog_password_remove_cb (GnomeKeyringResult result,
+			       gpointer data)
+{
+	GtkTreeRowReference *rowref = (GtkTreeRowReference *)data;
+
+	if (result == GNOME_KEYRING_RESULT_OK) {
+		GtkTreeIter iter;
+		GtkTreePath *path;
+		GtkTreeModel *model;
+
+		if (!gtk_tree_row_reference_valid (rowref))
+			return;
+
+		path = gtk_tree_row_reference_get_path (rowref);
+		model = gtk_tree_row_reference_get_model (rowref);
+
+		if (path != NULL && gtk_tree_model_get_iter (model, &iter, path)) {
+			gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
+			gtk_tree_path_free (path);
+		}
+	}
+}
+
+static void
+clear_all_dialog_response_cb (GtkDialog *dialog,
+			      int response,
+			      PdmClearAllDialogButtons *checkbuttons)
+{
+	if (response == GTK_RESPONSE_HELP)
+	{
+		/* Show help and return early */
+		ephy_gui_help (GTK_WIDGET (dialog), "clearing-personal-data");
+		return;
+	}
+
+	if (response == GTK_RESPONSE_OK)
+	{
+		if (gtk_toggle_button_get_active
+			(GTK_TOGGLE_BUTTON (checkbuttons->checkbutton_history)))
+		{
+			EphyEmbedShell *shell;
+			EphyHistory *history;
+
+			shell = ephy_embed_shell_get_default ();
+			history = EPHY_HISTORY (ephy_embed_shell_get_global_history (shell));
+			ephy_history_clear (history);
+		}
+		if (gtk_toggle_button_get_active
+			(GTK_TOGGLE_BUTTON (checkbuttons->checkbutton_cookies)))
+		{
+			SoupCookieJar *jar;
+
+			jar = get_cookie_jar ();
+			clear_all_cookies (jar);
+		}
+		if (gtk_toggle_button_get_active
+			(GTK_TOGGLE_BUTTON (checkbuttons->checkbutton_passwords)))
+		{
+			GtkTreeIter iter;
+			PdmDialog *pdialog = EPHY_PDM_DIALOG (checkbuttons->dialog);
+			PdmActionInfo *pinfo = pdialog->priv->passwords;
+
+			gboolean valid = gtk_tree_model_get_iter_first (pinfo->model, &iter);
+
+			while (valid) {
+				GtkTreePath *path;
+				EphyPasswordInfo *info;
+				GtkTreeRowReference *row;
+
+				path = gtk_tree_model_get_path (pinfo->model, &iter);
+				row = gtk_tree_row_reference_new (pinfo->model, path);
+
+				gtk_tree_model_get (pinfo->model, &iter,
+						    COL_PASSWORDS_DATA, &info,
+						    -1);
+
+				gnome_keyring_item_delete (GNOME_KEYRING_DEFAULT,
+							   info->keyring_id,
+							   (GnomeKeyringOperationDoneCallback) pdm_dialog_password_remove_cb,
+							   row,
+							   (GDestroyNotify) gtk_tree_row_reference_free);
+
+				valid = gtk_tree_model_iter_next (pinfo->model, &iter);
+
+				g_slice_free (EphyPasswordInfo, info);
+				gtk_tree_path_free (path);
+			}
+		}
+		if (gtk_toggle_button_get_active
+			(GTK_TOGGLE_BUTTON (checkbuttons->checkbutton_cache)))
+		{
+			EphyEmbedShell *shell;
+			EphyEmbedSingle *single;
+			EphyFaviconCache *cache;
+
+			shell = ephy_embed_shell_get_default ();
+
+			single = EPHY_EMBED_SINGLE (ephy_embed_shell_get_embed_single (shell));
+
+			ephy_embed_single_clear_cache (single);
+
+			cache = EPHY_FAVICON_CACHE (ephy_embed_shell_get_favicon_cache (shell));
+			ephy_favicon_cache_clear (cache);
+		}
+	}
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+}
+
+static void
+clear_all_dialog_checkbutton_toggled_cb (GtkToggleButton *toggle,
+					 PdmClearAllDialogButtons *data)
+{
+	GtkWidget *dialog;
+	dialog = gtk_widget_get_toplevel (GTK_WIDGET (toggle));
+
+	if (gtk_toggle_button_get_active (toggle) == TRUE)
+	{
+		data->num_checked++;
+	}
+	else
+	{
+		data->num_checked--;
+	}
+	gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog), GTK_RESPONSE_OK,
+					   data->num_checked != 0);
+}
+
+void
+pdm_dialog_show_clear_all_dialog (EphyDialog *edialog,
+				  GtkWidget *parent,
+				  PdmClearAllDialogFlags flags)
+{
+	GtkWidget *dialog, *vbox;
+	GtkWidget *check, *label;
+	PdmClearAllDialogButtons *checkbuttons;
+	GtkWidget *button, *icon;
+
+	dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW (parent),
+						     GTK_DIALOG_DESTROY_WITH_PARENT |
+						     GTK_DIALOG_MODAL,
+						     GTK_MESSAGE_QUESTION,
+						     GTK_BUTTONS_NONE,
+						     _("<b>Select the personal data "
+						       "you want to clear</b>"));
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+						  _("You are about to clear personal data "
+						    "that is stored about the web pages "
+						    "you have visited. Before proceeding, "
+						    "check the types of information that you "
+						    "want to remove:"));
+	gtk_window_set_title (GTK_WINDOW (dialog), _("Clear All Personal Data"));
+
+	gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+				GTK_STOCK_HELP,
+				GTK_RESPONSE_HELP,
+				GTK_STOCK_CANCEL,
+				GTK_RESPONSE_CANCEL,
+				NULL);
+
+	/* Clear button */
+	button = gtk_dialog_add_button (GTK_DIALOG (dialog),
+					_("Cl_ear"),
+					GTK_RESPONSE_OK);
+	icon = gtk_image_new_from_stock (GTK_STOCK_CLEAR, GTK_ICON_SIZE_BUTTON);
+	gtk_button_set_image (GTK_BUTTON (button), icon);
+	gtk_widget_show (button);
+
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
+					 GTK_RESPONSE_CANCEL);
+	gtk_label_set_selectable (GTK_LABEL (GTK_MESSAGE_DIALOG (dialog)->label),
+				  FALSE);
+
+	vbox = gtk_vbox_new (FALSE, 6);
+	gtk_box_pack_start (GTK_BOX (GTK_MESSAGE_DIALOG (dialog)->label->parent),
+			    vbox, FALSE, FALSE, 0);
+
+	checkbuttons = g_slice_new0 (PdmClearAllDialogButtons);
+	checkbuttons->dialog = edialog;
+	checkbuttons->num_checked = 0;
+
+	/* Cookies */
+	check = gtk_check_button_new_with_mnemonic (_("C_ookies"));
+	checkbuttons->checkbutton_cookies = check;
+	gtk_box_pack_start (GTK_BOX (vbox), check,
+			    FALSE, FALSE, 0);
+	g_signal_connect (check, "toggled",
+			  G_CALLBACK (clear_all_dialog_checkbutton_toggled_cb), checkbuttons);
+	if (flags & CLEAR_ALL_COOKIES)
+	{
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), TRUE);
+	}
+
+	/* Passwords */
+	check = gtk_check_button_new_with_mnemonic (_("Saved _passwords"));
+	checkbuttons->checkbutton_passwords = check;
+	gtk_box_pack_start (GTK_BOX (vbox), check,
+			    FALSE, FALSE, 0);
+	g_signal_connect (check, "toggled",
+			  G_CALLBACK (clear_all_dialog_checkbutton_toggled_cb), checkbuttons);
+	if (flags & CLEAR_ALL_PASSWORDS)
+	{
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), TRUE);
+	}
+
+	/* History */
+	check = gtk_check_button_new_with_mnemonic (_("Hi_story"));
+	checkbuttons->checkbutton_history = check;
+	gtk_box_pack_start (GTK_BOX (vbox), check,
+			    FALSE, FALSE, 0);
+	g_signal_connect (check, "toggled",
+			  G_CALLBACK (clear_all_dialog_checkbutton_toggled_cb), checkbuttons);
+	if (flags & CLEAR_ALL_HISTORY)
+	{
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), TRUE);
+	}
+
+	/* Cache */
+	check = gtk_check_button_new_with_mnemonic (_("_Temporary files"));
+	checkbuttons->checkbutton_cache = check;
+	gtk_box_pack_start (GTK_BOX (vbox), check,
+			    FALSE, FALSE, 0);
+	g_signal_connect (check, "toggled",
+			  G_CALLBACK (clear_all_dialog_checkbutton_toggled_cb), checkbuttons);
+	if (flags & CLEAR_ALL_CACHE)
+	{
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), TRUE);
+	}
+
+	/* Show vbox and all checkbuttons */
+	gtk_widget_show_all (vbox);
+
+	label = gtk_label_new (NULL);
+	gtk_label_set_markup (GTK_LABEL (label),
+			      _("<small><i><b>Note:</b> You cannot undo this action. "
+				"The data you are choosing to clear "
+				"will be deleted forever.</i></small>"));
+	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
+			    label, FALSE, FALSE, 0);
+	/* Need to do this or the label will wrap too early */
+	gtk_widget_set_size_request (label, 330, -1);
+	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+	gtk_misc_set_alignment (GTK_MISC (label),
+				0, 0);
+	gtk_misc_set_padding (GTK_MISC (label),
+			      6, 0);
+	gtk_widget_show (label);
+
+	gtk_window_present (GTK_WINDOW (dialog));
+	g_signal_connect_data (dialog, "response",
+			       G_CALLBACK (clear_all_dialog_response_cb),
+			       checkbuttons, (GClosureNotify) clear_all_dialog_release_cb,
+			       (GConnectFlags) 0);
 }
 
 static void
@@ -215,6 +490,7 @@ action_treeview_selection_changed_cb (GtkTreeSelection *selection,
 
 	widget = ephy_dialog_get_control (d, properties[action->remove_id].id);
 	gtk_widget_set_sensitive (widget, has_selection);
+
 }
 
 static void
@@ -245,14 +521,14 @@ pdm_cmd_delete_selection (PdmActionInfo *action)
 	}
 
 	/* Intelligent selection logic, no actual selection yet */
-	
-	path = gtk_tree_row_reference_get_path 
+
+	path = gtk_tree_row_reference_get_path
 		((GtkTreeRowReference *) g_list_first (rlist)->data);
-	
+
 	gtk_tree_model_get_iter (model, &iter, path);
 	gtk_tree_path_free (path);
 	iter2 = iter;
-	
+
 	if (gtk_tree_model_iter_next (GTK_TREE_MODEL (model), &iter))
 	{
 		path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), &iter);
@@ -267,16 +543,13 @@ pdm_cmd_delete_selection (PdmActionInfo *action)
 		}
 	}
 	gtk_tree_path_free (path);
-	
+
 	/* Removal */
-	
 	for (r = rlist; r != NULL; r = r->next)
 	{
 		GValue val = { 0, };
 
-		path = gtk_tree_row_reference_get_path
-			((GtkTreeRowReference *)r->data);
-
+		path = gtk_tree_row_reference_get_path ((GtkTreeRowReference *)r->data);
 		gtk_tree_model_get_iter (model, &iter, path);
 		gtk_tree_model_get_value (model, &iter, action->data_col, &val);
 		action->remove (action, g_value_get_boxed (&val));
@@ -297,7 +570,6 @@ pdm_cmd_delete_selection (PdmActionInfo *action)
 	g_list_free (rlist);
 
 	/* Selection */
-	
 	if (row_ref != NULL)
 	{
 		path = gtk_tree_row_reference_get_path (row_ref);
@@ -332,6 +604,9 @@ pdm_dialog_remove_button_clicked_cb (GtkWidget *button,
 				     PdmActionInfo *action)
 {
 	pdm_cmd_delete_selection (action);
+
+	/* Restore the focus to the button */
+	gtk_widget_grab_focus (button);
 }
 
 static void
@@ -355,20 +630,19 @@ setup_action (PdmActionInfo *action)
 			  "key_press_event",
 			  G_CALLBACK (pdm_key_pressed_cb),
 			  action);
-
 }
 
 /* "Cookies" tab */
 
 static void
 show_cookies_properties (PdmDialog *dialog,
-			 EphyCookie *info)
+			 SoupCookie *info)
 {
 	GtkWidget *gdialog;
 	GtkWidget *table;
 	GtkWidget *label;
 	GtkWidget *parent;
-	GtkWidget *dialog_vbox;
+	GtkWidget *content_area;
 	char *str;
 
 	parent = ephy_dialog_get_control (EPHY_DIALOG(dialog),
@@ -379,12 +653,13 @@ show_cookies_properties (PdmDialog *dialog,
 		  GTK_WINDOW (parent),
 		  GTK_DIALOG_DESTROY_WITH_PARENT,
 		  GTK_STOCK_CLOSE, 0, NULL);
-	ephy_state_add_window (GTK_WIDGET (gdialog), "cookie_properties", 
+	ephy_state_add_window (GTK_WIDGET (gdialog), "cookie_properties",
 			       -1, -1, FALSE,
 			       EPHY_STATE_WINDOW_SAVE_SIZE | EPHY_STATE_WINDOW_SAVE_POSITION);
 	gtk_dialog_set_has_separator (GTK_DIALOG(gdialog), FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (gdialog), 5);
-	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (gdialog)->vbox), 14); /* 24 = 2 * 5 + 14 */
+	content_area = gtk_dialog_get_content_area (GTK_DIALOG (gdialog));
+	gtk_box_set_spacing (GTK_BOX (content_area), 14); /* 24 = 2 * 5 + 14 */
 
 	table = gtk_table_new (2, 4, FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (table), 5);
@@ -433,7 +708,7 @@ show_cookies_properties (PdmDialog *dialog,
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3,
 			  GTK_FILL, GTK_FILL, 0, 0);
 
-	label = gtk_label_new (info->is_secure ? _("Encrypted connections only") : _("Any type of connection") );
+	label = gtk_label_new (info->secure ? _("Encrypted connections only") : _("Any type of connection") );
 	gtk_label_set_selectable (GTK_LABEL (label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
 	gtk_widget_show (label);
@@ -448,17 +723,16 @@ show_cookies_properties (PdmDialog *dialog,
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4,
 			  GTK_FILL, GTK_FILL, 0, 0);
 
-	if (info->is_session)
+	if (info->expires == NULL)
 	{
+		/* Session cookie */
 		str = g_strdup (_("End of current session"));
 	}
 	else
 	{
 		struct tm t;
-		char s[128];
-		const char *fmt_hack = "%c";
-		strftime (s, sizeof(s), fmt_hack, localtime_r (&info->expires, &t));
-		str = g_locale_to_utf8 (s, -1, NULL, NULL, NULL);
+		time_t out = soup_date_to_time_t(info->expires);
+		str = eel_strdup_strftime ("%c", localtime_r (&out, &t));
 	}
 	label = gtk_label_new (str);
 	g_free (str);
@@ -467,8 +741,7 @@ show_cookies_properties (PdmDialog *dialog,
 	gtk_widget_show (label);
 	gtk_table_attach_defaults (GTK_TABLE (table), label, 1, 2, 3, 4);
 
-	dialog_vbox = GTK_DIALOG(gdialog)->vbox;
-	gtk_box_pack_start (GTK_BOX(dialog_vbox),
+	gtk_box_pack_start (GTK_BOX (content_area),
 			    table,
 			    FALSE, FALSE, 0);
 
@@ -488,19 +761,18 @@ cookies_properties_clicked_cb (GtkWidget *button,
 	GValue val = {0, };
 	GtkTreeIter iter;
 	GtkTreePath *path;
-	EphyCookie *cookie;
+	SoupCookie *cookie;
 	GList *l;
 	GtkTreeSelection *selection;
 
 	selection = gtk_tree_view_get_selection (dialog->priv->cookies->treeview);
-	l = gtk_tree_selection_get_selected_rows
-		(selection, &model);
+	l = gtk_tree_selection_get_selected_rows (selection, &model);
 
 	path = (GtkTreePath *)l->data;
 	gtk_tree_model_get_iter (model, &iter, path);
 	gtk_tree_model_get_value
 		(model, &iter, COL_COOKIES_DATA, &val);
-	cookie = (EphyCookie *) g_value_get_boxed (&val);
+	cookie = (SoupCookie *) g_value_get_boxed (&val);
 
 	show_cookies_properties (dialog, cookie);
 
@@ -569,7 +841,7 @@ pdm_dialog_cookies_construct (PdmActionInfo *info)
 					G_TYPE_STRING,
 					G_TYPE_STRING,
 					G_TYPE_STRING,
-					EPHY_TYPE_COOKIE);
+					SOUP_TYPE_COOKIE);
 	gtk_tree_view_set_model (treeview, GTK_TREE_MODEL(liststore));
 	gtk_tree_view_set_headers_visible (treeview, TRUE);
 	selection = gtk_tree_view_get_selection (treeview);
@@ -620,20 +892,9 @@ pdm_dialog_cookies_construct (PdmActionInfo *info)
 	setup_action (info);
 }
 
-static gboolean
-compare_cookies (const EphyCookie *cookie1,
-		 const EphyCookie *cookie2)
-{
-	g_return_val_if_fail (cookie1 != NULL || cookie2 != NULL, FALSE);
-
-	return (strcmp (cookie1->domain, cookie2->domain) == 0
-		&& strcmp (cookie1->path, cookie2->path) == 0
-		&& strcmp (cookie1->name, cookie2->name) == 0);
-}
-
 static gboolean
 cookie_to_iter (GtkTreeModel *model,
-		const EphyCookie *cookie,
+		const SoupCookie *cookie,
 		GtkTreeIter *iter)
 {
 	gboolean valid;
@@ -643,15 +904,15 @@ cookie_to_iter (GtkTreeModel *model,
 
 	while (valid)
 	{
-		EphyCookie *data;
+		SoupCookie *data;
 
 		gtk_tree_model_get (model, iter,
 				    COL_COOKIES_DATA, &data,
 				    -1);
 
-		found = compare_cookies (cookie, data);
+		found = soup_cookie_equal ((SoupCookie*)cookie, data);
 
-		ephy_cookie_free (data);
+		soup_cookie_free (data);
 
 		if (found) break;
 
@@ -662,69 +923,43 @@ cookie_to_iter (GtkTreeModel *model,
 }
 
 static void
-cookie_added_cb (EphyCookieManager *manager,
-		 const EphyCookie *cookie,
-		 PdmDialog *dialog)
-{
-	PdmActionInfo *info = dialog->priv->cookies;
-	
-	LOG ("cookie_added_cb");
-
-	info->add (info, (gpointer) ephy_cookie_copy (cookie));
-}
-
-static void
-cookie_changed_cb (EphyCookieManager *manager,
-		   const EphyCookie *cookie,
+cookie_changed_cb (SoupCookieJar *jar,
+		   const SoupCookie *old_cookie,
+		   const SoupCookie *new_cookie,
 		   PdmDialog *dialog)
 {
-	PdmActionInfo *info = dialog->priv->cookies;
+	PdmActionInfo *info;
 	GtkTreeIter iter;
 
+	g_return_if_fail (EPHY_IS_PDM_DIALOG (dialog));
+	info = dialog->priv->cookies;
+
 	LOG ("cookie_changed_cb");
 
-	if (cookie_to_iter (info->model, cookie, &iter))
+	g_return_if_fail (info);
+
+	if (old_cookie)
 	{
-		gtk_list_store_remove (GTK_LIST_STORE (info->model), &iter);		
-		info->add (info, (gpointer) ephy_cookie_copy (cookie));
+		/* Cookie changed or deleted, let's get rid of the old one
+		   in any case */
+		if (cookie_to_iter (info->model, old_cookie, &iter))
+		{
+			gtk_list_store_remove (GTK_LIST_STORE (info->model), &iter);
+		}
+		else
+		{
+			g_warning ("Unable to find changed cookie in list!\n");
+		}
 	}
-	else
+
+	if (new_cookie)
 	{
-		g_warning ("Unable to find changed cookie in list!\n");
+		/* Cookie changed or added, let's add the new cookie in
+		   any case */
+		info->add (info, (gpointer) soup_cookie_copy ((SoupCookie*)new_cookie));
 	}
 }
 
-static void
-cookie_deleted_cb (EphyCookieManager *manager,
-		   const EphyCookie *cookie,
-		   PdmDialog *dialog)
-{
-	PdmActionInfo *info = dialog->priv->cookies;
-	GtkTreeIter iter;
-
-	LOG ("cookie_deleted_cb");
-
-	if (cookie_to_iter (info->model, cookie, &iter))
-	{
-		gtk_list_store_remove (GTK_LIST_STORE (info->model), &iter);		
-	}
-	else
-	{
-		g_warning ("Unable to find deleted cookie in list!\n");
-	}
-}
-
-static void
-cookies_cleared_cb (EphyCookieManager *manager,
-		    PdmDialog *dialog)
-{
-	PdmActionInfo *info = dialog->priv->cookies;
-
-	LOG ("cookies_cleared_cb");
-
-	gtk_list_store_clear (GTK_LIST_STORE (info->model));
-}
-
 static gboolean
 cookie_host_to_iter (GtkTreeModel *model,
 		     const char *key1,
@@ -754,7 +989,7 @@ cookie_host_to_iter (GtkTreeModel *model,
 		/* Start on the \0 */
 		p = key1 + len;
 		q = key2 + strlen (key2);
-	
+
 		do
 		{
 			if (*p == '.') ++n;
@@ -816,15 +1051,13 @@ compare_cookie_host_keys (GtkTreeModel *model,
 static void
 pdm_dialog_fill_cookies_list (PdmActionInfo *info)
 {
-	EphyCookieManager *manager;
-	GList *list, *l;
+	SoupCookieJar *jar;
+	GSList *list, *l;
 
 	g_assert (info->filled == FALSE);
 
-	manager = EPHY_COOKIE_MANAGER (ephy_embed_shell_get_embed_single
-			(EPHY_EMBED_SHELL (ephy_shell)));
-
-	list = ephy_cookie_manager_list_cookies (manager);
+	jar = get_cookie_jar ();
+	list = soup_cookie_jar_all_cookies (jar);
 
 	for (l = list; l != NULL; l = l->next)
 	{
@@ -832,7 +1065,7 @@ pdm_dialog_fill_cookies_list (PdmActionInfo *info)
 	}
 
 	/* the element data has been consumed, so we need only to free the list */
-	g_list_free (list);
+	g_slist_free (list);
 
 	/* Now turn on sorting */
 	gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (info->model),
@@ -842,18 +1075,11 @@ pdm_dialog_fill_cookies_list (PdmActionInfo *info)
 	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (info->model),
 					      COL_COOKIES_HOST_KEY,
 					      GTK_SORT_ASCENDING);
-	
+
 	info->filled = TRUE;
 
-	/* Now connect the callbacks on the EphyCookieManager */
-	g_signal_connect (manager, "cookie-added",
-			 G_CALLBACK (cookie_added_cb), info->dialog);
-	g_signal_connect (manager, "cookie-changed",
-			 G_CALLBACK (cookie_changed_cb), info->dialog);
-	g_signal_connect (manager, "cookie-deleted",
-			 G_CALLBACK (cookie_deleted_cb), info->dialog);
-	g_signal_connect (manager, "cookies-cleared",
-			 G_CALLBACK (cookies_cleared_cb), info->dialog);
+	g_signal_connect (jar, "changed",
+			  G_CALLBACK (cookie_changed_cb), info->dialog);
 
 	info->scroll_to (info);
 }
@@ -869,7 +1095,7 @@ static void
 pdm_dialog_cookie_add (PdmActionInfo *info,
 		       gpointer data)
 {
-	EphyCookie *cookie = (EphyCookie *) data;
+	SoupCookie *cookie = (SoupCookie *) data;
 	GtkListStore *store;
 	GtkTreeIter iter;
 	int column[4] = { COL_COOKIES_HOST, COL_COOKIES_HOST_KEY, COL_COOKIES_NAME, COL_COOKIES_DATA };
@@ -886,7 +1112,7 @@ pdm_dialog_cookie_add (PdmActionInfo *info,
 	g_value_init (&value[0], G_TYPE_STRING);
 	g_value_init (&value[1], G_TYPE_STRING);
 	g_value_init (&value[2], G_TYPE_STRING);
-	g_value_init (&value[3], EPHY_TYPE_COOKIE);
+	g_value_init (&value[3], SOUP_TYPE_COOKIE);
 
 	g_value_set_static_string (&value[0], cookie->domain);
 	g_value_take_string (&value[1], ephy_string_collate_key_for_domain (cookie->domain, -1));
@@ -907,13 +1133,9 @@ static void
 pdm_dialog_cookie_remove (PdmActionInfo *info,
 			  gpointer data)
 {
-	EphyCookie *cookie = (EphyCookie *) data;
-	EphyCookieManager *manager;
+	SoupCookie *cookie = (SoupCookie *) data;
 
-	manager = EPHY_COOKIE_MANAGER (ephy_embed_shell_get_embed_single
-			(EPHY_EMBED_SHELL (ephy_shell)));
-
-	ephy_cookie_manager_remove_cookie (manager, cookie);
+	soup_cookie_jar_delete_cookie (get_cookie_jar(), cookie);
 }
 
 static void
@@ -940,6 +1162,70 @@ pdm_dialog_cookie_scroll_to (PdmActionInfo *info)
 }
 
 /* "Passwords" tab */
+static void
+passwords_data_func_get_item_cb (GnomeKeyringResult result,
+				 GnomeKeyringItemInfo *info,
+				 gpointer data)
+{
+	GtkTreeRowReference *rowref = (GtkTreeRowReference *)data;
+
+	if (result == GNOME_KEYRING_RESULT_OK) {
+		GtkTreeIter iter;
+		GtkTreePath *path;
+		GtkTreeModel *model;
+
+		if (!gtk_tree_row_reference_valid (rowref))
+			return;
+
+		path = gtk_tree_row_reference_get_path (rowref);
+		model = gtk_tree_row_reference_get_model (rowref);
+
+		if (path != NULL && gtk_tree_model_get_iter (model, &iter, path)) {
+			EphyPasswordInfo *epinfo;
+			GValue val = {0, };
+
+			gtk_tree_model_get_value (model, &iter, COL_PASSWORDS_DATA, &val);
+			epinfo = g_value_get_boxed (&val);
+			epinfo->secret = gnome_keyring_memory_strdup (gnome_keyring_item_info_get_secret (info));
+
+			gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+					    COL_PASSWORDS_DATA, epinfo,
+					    COL_PASSWORDS_PASSWORD, epinfo->secret, -1);
+			g_value_unset (&val);
+		}
+	}
+}
+
+static void
+passwords_data_func (GtkTreeViewColumn *tree_column, GtkCellRenderer *cell,
+		     GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
+{
+	GValue val = { 0, };
+	EphyPasswordInfo *info;
+
+	if (!gtk_tree_view_column_get_visible (tree_column))
+		return;
+
+	gtk_tree_model_get_value (model, iter, COL_PASSWORDS_DATA, &val);
+	info = g_value_get_boxed (&val);
+
+	if (info->secret == NULL) {
+		GtkTreePath *path;
+		GtkTreeRowReference *rowref;
+
+		path = gtk_tree_model_get_path (model, iter);
+		rowref = gtk_tree_row_reference_new (model, path);
+
+		gnome_keyring_item_get_info_full (GNOME_KEYRING_DEFAULT,
+						  info->keyring_id,
+						  GNOME_KEYRING_ITEM_INFO_SECRET,
+						  (GnomeKeyringOperationGetItemInfoCallback) passwords_data_func_get_item_cb,
+						  rowref,
+						  (GDestroyNotify) gtk_tree_row_reference_free);
+		gtk_tree_path_free (path);
+	}
+	g_value_unset (&val);
+}
 
 static void
 passwords_show_toggled_cb (GtkWidget *button,
@@ -954,7 +1240,7 @@ passwords_show_toggled_cb (GtkWidget *button,
 	column = gtk_tree_view_get_column (treeview, COL_PASSWORDS_PASSWORD);
 
 	active = gtk_toggle_button_get_active ((GTK_TOGGLE_BUTTON (button)));
-	
+
 	gtk_tree_view_column_set_visible (column, active);
 }
 
@@ -985,6 +1271,7 @@ pdm_dialog_passwords_construct (PdmActionInfo *info)
 					G_TYPE_STRING,
 					G_TYPE_STRING,
 					EPHY_TYPE_PASSWORD_INFO);
+
 	gtk_tree_view_set_model (treeview, GTK_TREE_MODEL(liststore));
 	gtk_tree_view_set_headers_visible (treeview, TRUE);
 	selection = gtk_tree_view_get_selection (treeview);
@@ -1007,6 +1294,7 @@ pdm_dialog_passwords_construct (PdmActionInfo *info)
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 	gtk_tree_view_column_set_sort_column_id (column, COL_PASSWORDS_HOST);
 
+	renderer = gtk_cell_renderer_text_new ();
 	gtk_tree_view_insert_column_with_attributes (treeview,
 						     COL_PASSWORDS_USER,
 						     _("User Name"),
@@ -1019,6 +1307,7 @@ pdm_dialog_passwords_construct (PdmActionInfo *info)
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 	gtk_tree_view_column_set_sort_column_id (column, COL_PASSWORDS_USER);
 
+	renderer = gtk_cell_renderer_text_new ();
 	gtk_tree_view_insert_column_with_attributes (treeview,
 						     COL_PASSWORDS_PASSWORD,
 						     _("User Password"),
@@ -1026,70 +1315,57 @@ pdm_dialog_passwords_construct (PdmActionInfo *info)
 						     "text", COL_PASSWORDS_PASSWORD,
 						     NULL);
 	column = gtk_tree_view_get_column (treeview, COL_PASSWORDS_PASSWORD);
-	/* Hide this info by default */
+	gtk_tree_view_column_set_cell_data_func (column,
+						 renderer,
+						 passwords_data_func,
+						 info,
+						 NULL);
+	/* Initially shown as hidden colum */
 	gtk_tree_view_column_set_visible (column, FALSE);
 	gtk_tree_view_column_set_resizable (column, TRUE);
 	gtk_tree_view_column_set_reorderable (column, TRUE);
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
-	gtk_tree_view_column_set_sort_column_id (column, COL_PASSWORDS_PASSWORD);
 
 	info->treeview = treeview;
-
 	setup_action (info);
 }
 
 static void
-passwords_changed_cb (EphyPasswordManager *manager,
-		      PdmDialog *dialog)
+pdm_dialog_fill_passwords_list_async_cb (GnomeKeyringResult result,
+					 GList *list,
+					 gpointer data)
 {
-	GtkTreeModel *model = dialog->priv->passwords->model;
+	GList *l;
+	PdmActionInfo *info = (PdmActionInfo *)data;
 
-	LOG ("passwords changed");
+	if (result != GNOME_KEYRING_RESULT_OK)
+		return;
 
-	/* since the callback doesn't carry any information about what
-	 * exactly has changed, we have to rebuild the list from scratch.
-	 */
-	gtk_list_store_clear (GTK_LIST_STORE (model));
+	for (l = list; l != NULL; l = l->next)
+		info->add (info, l->data);
 
-	/* And turn off sorting */
-	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (model),
-					      GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID,
+	info->filled = TRUE;
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (info->model),
+					      COL_PASSWORDS_HOST,
 					      GTK_SORT_ASCENDING);
-
-	dialog->priv->passwords->fill (dialog->priv->passwords);
 }
 
 static void
 pdm_dialog_fill_passwords_list (PdmActionInfo *info)
 {
-	EphyPasswordManager *manager;
-	GList *list, *l;
+	gnome_keyring_list_item_ids (GNOME_KEYRING_DEFAULT,
+				     pdm_dialog_fill_passwords_list_async_cb,
+				     info,
+				     NULL);
+}
 
-	manager = EPHY_PASSWORD_MANAGER (ephy_embed_shell_get_embed_single
-			(EPHY_EMBED_SHELL (ephy_shell)));
+static void
+pdm_dialog_fill_passwords_list_from_soupsession_dummy (PdmActionInfo *info)
+{
+	/* Do nothing, no HTTP password will be added to PDM (in private mode) because we cant delete them.
 
-	list = ephy_password_manager_list_passwords (manager);
-
-	for (l = list; l != NULL; l = l->next)
-	{
-		info->add (info, l->data);
-	}
-
-	/* the element data has been consumed, so we need only to free the list */
-	g_list_free (list);
-
-	/* Let's get notified when the list changes */
-	if (info->filled == FALSE)
-	{
-		g_signal_connect (manager, "passwords-changed",
-				  G_CALLBACK (passwords_changed_cb), info->dialog);
-	}
-
-	info->filled = TRUE;
-
-	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (info->model),
-					      COL_PASSWORDS_HOST,
-					      GTK_SORT_ASCENDING);
+	   See https://bugzilla.gnome.org/show_bug.cgi?id=591395
+	*/
 }
 
 static void
@@ -1097,29 +1373,102 @@ pdm_dialog_passwords_destruct (PdmActionInfo *info)
 {
 }
 
+static void
+pdm_dialog_password_add_item_attrs_cb (GnomeKeyringResult result,
+				       GnomeKeyringAttributeList *attributes,
+				       gpointer data)
+{
+	EphyPasswordInfo *pinfo;
+	PdmCallBackData *cbdata;
+	GnomeKeyringAttribute *attribute;
+	gchar *user, *host, *protocol;
+	GtkTreeIter iter;
+	int i;
+
+	if (result != GNOME_KEYRING_RESULT_OK)
+		return;
+
+	cbdata = (PdmCallBackData *)data;
+
+	user = host = protocol = NULL;
+	attribute = (GnomeKeyringAttribute *) attributes->data;
+	for (i = 0; i < attributes->len; ++i) {
+		if (attribute[i].type == GNOME_KEYRING_ATTRIBUTE_TYPE_STRING) {
+			if (strcmp (attribute[i].name, "server") == 0)
+				host = g_strdup (attribute[i].value.string);
+			else if (strcmp (attribute[i].name, "user") == 0)
+				user = g_strdup (attribute[i].value.string);
+			else if (strcmp (attribute[i].name, "protocol") == 0)
+				protocol = attribute[i].value.string;
+			}
+		}
+	if (!protocol || strncmp("http", protocol, 4) != 0)
+		return;
+
+	pinfo = ephy_password_info_new (cbdata->key);
+	if (!pinfo)
+		return;
+
+	gtk_list_store_append (cbdata->store, &iter);
+	gtk_list_store_set (cbdata->store, &iter,
+			    COL_PASSWORDS_HOST, host,
+			    COL_PASSWORDS_USER, user,
+			    COL_PASSWORDS_PASSWORD, NULL,
+			    COL_PASSWORDS_DATA, pinfo,
+			    -1);
+}
+
+static void
+pdm_dialog_password_add_item_info_cb (GnomeKeyringResult result,
+				      GnomeKeyringItemInfo *info,
+				      gpointer data)
+{
+	if (result != GNOME_KEYRING_RESULT_OK)
+		return;
+
+	if (gnome_keyring_item_info_get_type (info) == GNOME_KEYRING_ITEM_NETWORK_PASSWORD) {
+		PdmCallBackData *cbdata = (PdmCallBackData *)data;
+		gnome_keyring_item_get_attributes (GNOME_KEYRING_DEFAULT,
+						   cbdata->key,
+						   (GnomeKeyringOperationGetAttributesCallback) pdm_dialog_password_add_item_attrs_cb,
+						   g_memdup (cbdata, sizeof (PdmCallBackData)),
+						   (GDestroyNotify) g_free);
+
+	}
+
+}
+
 static void
 pdm_dialog_password_add (PdmActionInfo *info,
 			 gpointer data)
 {
-	EphyPasswordInfo *pinfo = (EphyPasswordInfo *) data;
-	GtkListStore *store;
-	GtkTreeIter iter;
-	GValue value = { 0, };
+	PdmCallBackData *cbdata;
+	guint key_id = GPOINTER_TO_UINT(data);
 
-	store = GTK_LIST_STORE (info->model);
+	/*
+	 * We have the item id of the password. We will have to check if this
+	 * password entry is of the right type and then can proceed to get the
+	 * the private information. Seahorse is treating every protocol that
+	 * starts with http as Web Access and we will do the same here.
+	 */
 
-	gtk_list_store_append (store, &iter);
-	gtk_list_store_set (store,
-			    &iter,
-			    COL_PASSWORDS_HOST, pinfo->host,
-			    COL_PASSWORDS_USER, pinfo->username,
-			    COL_PASSWORDS_PASSWORD, pinfo->password,
-			    -1);
+	cbdata = g_malloc (sizeof (PdmCallBackData));
+	cbdata->key = key_id;
+	cbdata->store = GTK_LIST_STORE (info->model);
 
-	g_value_init (&value, EPHY_TYPE_PASSWORD_INFO);
-	g_value_take_boxed (&value, pinfo);
-	gtk_list_store_set_value (store, &iter, COL_PASSWORDS_DATA, &value);
-	g_value_unset (&value);
+	/* Get the type of the key_id */
+	gnome_keyring_item_get_info_full (GNOME_KEYRING_DEFAULT,
+					  key_id,
+					  GNOME_KEYRING_ITEM_INFO_BASICS,
+					  (GnomeKeyringOperationGetItemInfoCallback) pdm_dialog_password_add_item_info_cb,
+					  cbdata,
+					  (GDestroyNotify) g_free);
+}
+
+static void
+pdm_dialog_password_remove_dummy_cb (GnomeKeyringResult result,
+				     gpointer data)
+{
 }
 
 static void
@@ -1127,23 +1476,12 @@ pdm_dialog_password_remove (PdmActionInfo *info,
 			    gpointer data)
 {
 	EphyPasswordInfo *pinfo = (EphyPasswordInfo *) data;
-	EphyPasswordManager *manager;
 
-	manager = EPHY_PASSWORD_MANAGER (ephy_embed_shell_get_embed_single
-			(EPHY_EMBED_SHELL (ephy_shell)));
-
-	/* we don't remove the password from the liststore in the callback
-	 * like we do for cookies, since the callback doesn't carry that
-	 * information, and we'd have to reload the whole list, losing the
-	 * selection in the process.
-	 */
-	g_signal_handlers_block_by_func
-		(manager, G_CALLBACK (passwords_changed_cb), info->dialog);
-
-	ephy_password_manager_remove_password (manager, pinfo);
-
-	g_signal_handlers_unblock_by_func
-		(manager, G_CALLBACK (passwords_changed_cb), info->dialog);
+	gnome_keyring_item_delete (GNOME_KEYRING_DEFAULT,
+				   pinfo->keyring_id,
+				   (GnomeKeyringOperationDoneCallback) pdm_dialog_password_remove_dummy_cb,
+				   NULL,
+				   NULL);
 }
 
 /* common routines */
@@ -1160,7 +1498,6 @@ sync_notebook_tab (GtkWidget *notebook,
 	if (page_num == 0 && priv->cookies->filled == FALSE)
 	{
 		priv->cookies->fill (priv->cookies);
-
 		priv->cookies->scroll_to (priv->cookies);
 	}
 	else if (page_num == 1 && priv->passwords->filled == FALSE)
@@ -1179,6 +1516,29 @@ pdm_dialog_response_cb (GtkDialog *widget,
 		pdm_dialog_show_help (dialog);
 		return;
 	}
+	if (response == PDM_DIALOG_RESPONSE_CLEAR)
+	{
+		int page;
+		GtkWidget *parent;
+
+		parent = ephy_dialog_get_control (EPHY_DIALOG (dialog),
+						  properties[PROP_WINDOW].id);
+
+		page = gtk_notebook_get_current_page (GTK_NOTEBOOK (dialog->priv->notebook));
+		if (page == 0)
+		{
+			/* Cookies */
+			pdm_dialog_show_clear_all_dialog (EPHY_DIALOG (dialog),
+							  parent, CLEAR_ALL_COOKIES);
+		}
+		if (page == 1)
+		{
+			/* Passwords */
+			pdm_dialog_show_clear_all_dialog (EPHY_DIALOG (dialog),
+							  parent, CLEAR_ALL_PASSWORDS);
+		}
+		return;
+	}
 
 	g_object_unref (dialog);
 }
@@ -1189,12 +1549,14 @@ pdm_dialog_init (PdmDialog *dialog)
 	PdmDialogPrivate *priv;
 	PdmActionInfo *cookies, *passwords;
 	GtkWidget *window;
+	gboolean has_private_profile = ephy_has_private_profile ();
+
 
 	priv = dialog->priv = EPHY_PDM_DIALOG_GET_PRIVATE (dialog);
 
 	ephy_dialog_construct (EPHY_DIALOG(dialog),
 			       properties,
-			       ephy_file ("epiphany.glade"),
+			       ephy_file ("epiphany.ui"),
 			       "pdm_dialog",
 			       NULL);
 
@@ -1205,11 +1567,11 @@ pdm_dialog_init (PdmDialog *dialog)
 
 	ephy_gui_ensure_window_group (GTK_WINDOW (window));
 
-	gtk_window_set_icon_name (GTK_WINDOW (window), "web-browser");
+	gtk_window_set_icon_name (GTK_WINDOW (window), EPHY_STOCK_EPHY);
 
 	g_signal_connect (window, "response",
 			  G_CALLBACK (pdm_dialog_response_cb), dialog);
-	/**
+	/*
 	 * Group all Properties and Remove buttons in the same size group to
 	 * avoid the little jerk you get otherwise when switching pages because
 	 * one set of buttons is wider than another.
@@ -1237,7 +1599,8 @@ pdm_dialog_init (PdmDialog *dialog)
 	passwords = g_new0 (PdmActionInfo, 1);
 	passwords->construct = pdm_dialog_passwords_construct;
 	passwords->destruct = pdm_dialog_passwords_destruct;
-	passwords->fill = pdm_dialog_fill_passwords_list;
+	/* Bug 591395 : we dont show HTTP auth in private mode because we can delete or use it anyway.  */
+	passwords->fill = has_private_profile ? pdm_dialog_fill_passwords_list_from_soupsession_dummy : pdm_dialog_fill_passwords_list;
 	passwords->add = pdm_dialog_password_add;
 	passwords->remove = pdm_dialog_password_remove;
 	passwords->dialog = dialog;
@@ -1269,13 +1632,15 @@ pdm_dialog_finalize (GObject *object)
 	g_signal_handlers_disconnect_matched
 		(single, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, object);
 
+	g_signal_handlers_disconnect_by_func (get_cookie_jar (), cookie_changed_cb, object);
+
 	dialog->priv->cookies->destruct (dialog->priv->cookies);
 	dialog->priv->passwords->destruct (dialog->priv->passwords);
 
 	g_free (dialog->priv->passwords);
 	g_free (dialog->priv->cookies);
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (pdm_dialog_parent_class)->finalize (object);
 }
 
 void
